
# https://www.exceptionalcreatures.com/guides/advanced-rescue-and-raise

module DM::Bevel


  class BevelError < StandardError

    # @return [Object, Nil]
    attr_reader :source
    # @return [Object, Nil]
    attr_reader :cause

    # @param msg [String]
    # @param source [Object]
    # @param cause [Object]
    # @param data [Hash]
    def initialize(msg = nil, source = nil, cause = nil, **data)
      super(msg)

      @source = source
      @cause = cause
      @data = data.clone
    end

  end



  # Used for debugging (Should never be visible to actual users)
  class BevelDebugError < BevelError
  end

  class UnexpectedTypeError < BevelDebugError
  end

  class UnexpectedStateError < BevelDebugError
  end

  # User-friendly errors (Errors that should be captured by the tool and display a user-friendly message)
  class BevelUserError < BevelError
  end

  class EdgeOffsetTooSmallError < BevelUserError
  end

  class BevelEntitiesError < BevelUserError
  end

  class InvalidEdgeError < BevelUserError

    class Source
      EDGE = 1
      FACE = 2
      VERTEX = 3
      SURFACE = 4
    end

    # @return [BevelableEdgeState]
    attr_reader :edge_state

    # @param edge_state [BevelableEdgeState]
    def initialize(edge_state = nil)
      super()

      @edge_state = edge_state
      @source = Source::EDGE
    end

    # @param val [Source]
    def source=(val)
      return unless [Source::EDGE, Source::FACE, Source::VERTEX, Source::SURFACE].include?(val)

      @source = val
    end

    # @return [Integer] A Source value which is just an Integer.
    def source
      @source
    end

    # @param contains_state [Integer, BevelableEdgeState]
    # @return [Boolean]
    def contains?(contains_state)
      self.edge_state & contains_state == contains_state
    end


  end



end