
module DM::Bevel

  class BevelParams

    class Defaults
      SEGMENTS = 6
      EDGE_OFFSET = 12.0
      SOFTEN_ANGLE = 30.0 # degrees
      SMOOTH_NORMALS = true
      SMOOTH_PERIMETER = true
    end

    MAX_SEGMENTS = 24

    # @return [Integer]
    attr_reader :segments
    # @return [Float]
    attr_reader :edge_offset
    # @return [Float]
    attr_reader :soften_angle

    def segments=(val)
      val = val.to_i.abs
      val = MAX_SEGMENTS if val > MAX_SEGMENTS
      if val == 1
        @segments = val
      else
        if val < 2
          val = 2
        else
          val -= 1 unless val % 2 == 0
        end
        @segments = val
      end
    end

    def edge_offset=(val)
      offset = val.to_f.abs
      offset = 0.0 if offset <= 0.001 # Enforce minimum value
      @edge_offset = offset
    end

    def soften_angle=(val)
      val = val.to_f
      if val < 0
        val = 0.0
      elsif val > 180
        val = 180.0
      end
      @soften_angle = val
    end

    # @return [Boolean]
    def smooth_normals?
      @smooth_normals
    end

    def smooth_normals=(val)
      @smooth_normals = val == true
    end

    # @return [Boolean]
    def smooth_perimeter?
      @smooth_perimeter
    end

    def smooth_perimeter=(val)
      @smooth_perimeter = val == true
    end

    # @return [Boolean]
    def single_segment?
      @segments == 1
    end

    def initialize
      @segments = Defaults::SEGMENTS
      @edge_offset = Defaults::EDGE_OFFSET
      @soften_angle = Defaults::SOFTEN_ANGLE
      @smooth_normals = Defaults::SMOOTH_NORMALS
      @smooth_perimeter = Defaults::SMOOTH_PERIMETER
    end

    def restore_defaults!
      params = BevelParams.new
      self.instance_variables.each do |sym|
        self.instance_variable_set(sym, params.instance_variable_get(sym))
      end
    end

    # @param other_params [BevelParams]
    # @return [Boolean]
    def ==(other_params)
      return false unless other_params.instance_of?(BevelParams)

      return false unless (self.edge_offset - other_params.edge_offset).round(PLUGIN::FLOAT_PRECISION).zero?
      return false unless self.segments == other_params.segments
      return false unless self.smooth_perimeter? == other_params.smooth_perimeter?
      return false unless self.smooth_normals? == other_params.smooth_normals?
      return false unless self.soften_angle == other_params.soften_angle
      true
    end

  end

end