
module DM::Bevel

  class BevelSettingsDialog < Window
    extend Singleton

    # HTML
    ID_SMOOTH_PERIMETER = 'smooth_perimeter'
    ID_LANGUAGE = 'language'
    ID_SOFTEN_ANGLE = "soften_angle"
    ID_SMOOTH_NORMALS = "smooth_normals"
    ID_SHOW_NOTIFICATION = "show_notification"

    def initialize
      options = {
        :dialog_title => LH["Bevel Preferences"],
        :preferences_key => PLUGIN_NAME + "Settings",
        :scrollable => true,
        :resizable => true,
        :width => 330,
        :height => 330,
        :left => 50,
        :top => 300,
        :min_width => 300,
        :min_height => 310,
        :max_width => 800,
        :max_height => 400,
        :style => UI::HtmlDialog::STYLE_UTILITY
      }
      @dialog_filename = "bevel_settings.html"
      super(options)
    end

    def add_callbacks
      super
      add_action_callback('change_soften_angle') do |_, is_increment|
        deg = self.soften_angle
        incr = 15.0
        deg += is_increment == true ? incr : -incr
        self.soften_angle = deg
      end
    end

    # Needed so the dialog is 'refreshed' / shown after it is closed
    def show
      if visible?
        bring_to_front
      else
        initialize
        super
      end
    end

    def refresh_dialog
      set_value(ID_SOFTEN_ANGLE, self.soften_angle.to_s)
      set_checkbox(ID_SMOOTH_NORMALS, self.smooth_normals?)
      set_checkbox(ID_SMOOTH_PERIMETER, self.smooth_perimeter?)
      set_checkbox(ID_SHOW_NOTIFICATION, self.show_notification?)
      load_language
    end

    def language=(val)
      LH.language_file = val
      UI.messagebox(LH["SketchUp must be restarted for new language settings to take effect."])
    end

    def load_language
      clear_list(ID_LANGUAGE)
      LH.available_languages.each do |language|
        create_option(ID_LANGUAGE, language, language)
      end
      set_value(ID_LANGUAGE, LH.language_file)
    end

    def soften_angle=(val)
      deg = val.to_f.round(1)
      deg = 0.0 if deg < 0.0
      deg = 180.0 if deg > 180.0
      Sketchup.write_default(MAIN_DICTIONARY, ID_SOFTEN_ANGLE, deg)
      set_value(ID_SOFTEN_ANGLE, deg)
    end

    def soften_angle
      Sketchup.read_default(MAIN_DICTIONARY, ID_SOFTEN_ANGLE, BevelParams::Defaults::SOFTEN_ANGLE)
    end

    # @param [Boolean]
    def smooth_normals=(val)
      bool = val.downcase == 'true'
      Sketchup.write_default(MAIN_DICTIONARY, ID_SMOOTH_NORMALS, bool)
    end

    def smooth_normals?
      Sketchup.read_default(MAIN_DICTIONARY, ID_SMOOTH_NORMALS, BevelParams::Defaults::SMOOTH_NORMALS)
    end

    def smooth_perimeter=(val)
      val = val.downcase == 'true'

      Sketchup.write_default(MAIN_DICTIONARY, ID_SMOOTH_PERIMETER, val)
      # Refresh to tool
      if CoreTool.active_tool.is_a?(BaseTool)
        BaseTool.active_tool.bevel_params.smooth_perimeter = val
        Sketchup.active_model.active_view.invalidate
      end
    end

    # @return [Boolean]
    def smooth_perimeter?
      Sketchup.read_default(MAIN_DICTIONARY, ID_SMOOTH_PERIMETER, BevelParams::Defaults::SMOOTH_PERIMETER)
    end

    def show_notification=(val)
      val = val.downcase == 'true'
      Sketchup.write_default(MAIN_DICTIONARY, ID_SHOW_NOTIFICATION, val)
    end

    # Note: Applies only to Primary-Secondary cases.
    # @return [Boolean]
    def show_notification?
      Sketchup.read_default(MAIN_DICTIONARY, ID_SHOW_NOTIFICATION, true)
    end

  end

end