
module DM::Bevel

  class CenterPatchData
    # @return [Geom::Point3d]
    attr_reader :center_pt
    # @return [Hash{TransitionSegment => Array<Geom::Point3d>}]
    attr_reader :transition_segment_to_surrounding_pts

    # @param center_pt [Array<Geom::Point3d>]
    # @param transition_segment_to_surrounding_pts [Hash{TransitionSegment => Array<Geom::Point3d>}]
    def initialize(center_pt, transition_segment_to_surrounding_pts)
      # @type [Geom::Point3d]
      @center_pt = center_pt
      # @type [Hash{TransitionSegment => Array<Geom::Point3d>}]
      @transition_segment_to_surrounding_pts = transition_segment_to_surrounding_pts
    end

    # @return [Array<Geom::Point3d>]
    def surrounding_pts
      surrounding_pts = []
      @transition_segment_to_surrounding_pts.each_value do |pts|
        pts.each do |pt|
          surrounding_pts << pt unless surrounding_pts.include?(pt)
        end
      end
      surrounding_pts
    end

    # @return [Boolean]
    def face_properties_different?
      @transition_segment_to_surrounding_pts.each_key do |transition_segment|
        return true if transition_segment.face_properties_different?
      end
      false
    end

  end

  class CornerPatch

    # @return [TransitionLoop]
    attr_reader :transition_loop
    # @return [CenterPatchData]
    attr_reader :center_patch
    # @return [Hash{TransitionSegment => Array<PointCurve>}]
    attr_reader :transition_segment_to_point_curves

    # @param transition_loop [TransitionLoop]
    # @param transition_segment_to_point_curves [Hash{TransitionSegment => Array<PointCurve>}]
    # @param center_patch [CenterPatchData]
    def initialize(transition_loop, transition_segment_to_point_curves, center_patch)
      transition_segment_to_point_curves.each_key do |t_s|
        raise UnexpectedStateError, "TransitionLoop does not include the TransitionSegment" unless transition_loop.transition_segments.include?(t_s)
      end
      # @type [TransitionLoop]
      @transition_loop = transition_loop
      # @type [Hash{TransitionSegment => Array<PointCurve>}]
      @transition_segment_to_point_curves = transition_segment_to_point_curves
      # @type [CenterPatchData]
      @center_patch = center_patch
    end


    def inspect
      self.to_s
    end

  end

end



