
module DM::Bevel

  class ParallelFaceCorner < FaceCorner

    # @param edge_vert1 [EdgeVertex]
    # @param edge_vert2 [EdgeVertex]
    # @param face [Sketchup::Face]
    def initialize(edge_vert1, edge_vert2, face)
      raise UnexpectedStateError, "edge_verts are not parallel" unless edge_vert1.parallel?(edge_vert2)
      raise UnexpectedStateError, "edge_verts do not share the face" if !edge_vert1.faces.include?(face) || !edge_vert2.faces.include?(face)

      super(edge_vert1, edge_vert2)
      @face = face
    end

    # @return [Sketchup::Face]
    def face
      @face
    end

    # @return [Sketchup::Face]
    def faces
      [self.face]
    end

    def face_between?
      raise UnexpectedStateError.new("Does not apply when faces are parallel", self)
    end

    # The vector in the direction of the Face.
    def mid_vector
      face_vec =  self.edge_vert2.edge_vector * self.face.normal
      # Make sure the vector is in the shared face direction.
      pt = self.vertex.position.offset(face_vec, 0.01)
      face_vec.reverse! unless self.point_on_face?(self.face, pt)
      face_vec
    end

    # @param other_face_corner [FaceCorner]
    # @return [Boolean]
    def ==(other_face_corner)
      return false unless super
      self.face == other_face_corner.face
    end

    alias eql? :==

    def hash
      @edge_vert1.hash + @edge_vert2.hash + @face.hash
    end

    def inspect
      self.to_s
    end

  end

end