
module DM::Bevel

  class QuadFaceCorner < FaceCorner

    # Returns the FaceCorner that aligns with self.edge_vert1
    # @return [FaceCorner]
    def face_corner1
      @inner_face_corners[0]
    end

    # Returns the FaceCorner that aligns with self.edge_vert2
    # @return [FaceCorner]
    def face_corner2
      @inner_face_corners[1]
    end

    # @return [Array<FaceCorner>]
    def inner_face_corners
      @inner_face_corners.clone
    end

    # @return [EdgeVertex]
    def quad_diagonal_edge_vert
      @inner_face_corners[0].edge_vert2
    end

    # @param face_corner1 [FaceCorner]
    # @param face_corner2 [FaceCorner]
    def initialize(face_corner1, face_corner2)
      raise UnexpectedStateError, "face_corner1.edge_vert2 must be the same as face_corner2.edge_vert1" if face_corner1.edge_vert2 != face_corner2.edge_vert1
      raise UnexpectedStateError, "Must be a valid quad" unless face_corner1.edge_vert2.quadface_diagonal?

      super(face_corner1.edge_vert1, face_corner2.edge_vert2)
      # @type [Array<FaceCorner>]
      @inner_face_corners = [face_corner1, face_corner2]
      @quad_face = QuadFace.new(face_corner1.face, face_corner1.edge_vert2)
    end

    def faces
      [@quad_face]
    end

    def mid_vector
      val = @cache_manager.retrieve(:mid_vector) do
        # Use the calculated Quad plane as reference for where the points should lie.
        plane = self.face.plane
        vert_pt = self.vertex.position.project_to_plane(plane)
        vec1  = vert_pt.vector_to(self.edge_vert1.opposite_vertex.position.project_to_plane(plane)).normalize
        vec2  = vert_pt.vector_to(self.edge_vert2.opposite_vertex.position.project_to_plane(plane)).normalize

        vec = self.linear_combination(vec1, vec2).normalize
        vec.reverse! unless self.similar_direction?(vec, self.quad_diagonal_edge_vert.edge_vector)
        vec
      end
      val.clone
    end

    def face_between?
      val = @cache_manager.retrieve(:face_between) do
        self.similar_direction?(self.mid_vector, self.quad_diagonal_edge_vert.edge_vector)
      end
      val
    end

    def three_point_corner?
      false # Don't allow this for this case
    end

    # @param edge_vert [EdgeVertex]
    # @return [FaceCorner]
    def closest_face_corner(edge_vert)
      return nil unless self.edge_verts.include?(edge_vert)

      if self.edge_vert1 == edge_vert
        self.face_corner1
      else
        self.face_corner2
      end
    end

    def hash
      @edge_vert1.hash + @edge_vert2.hash + self.quad_diagonal_edge_vert.hash
    end


  end

end