
module DM::Bevel

  class FakeFace

    # @return [Sketchup::Vertex]
    attr_reader :vertex

    # @return [Geom::Vector3d]
    def normal
      @normal.clone
    end

    # @param vertex [Sketchup::Vertex]
    # @param normal [Geom::Vector3d]
    def initialize(vertex, normal)
      raise UnexpectedTypeError, "vertex must be of type Sketchup::Vertex" unless vertex.instance_of?(Sketchup::Vertex)
      raise UnexpectedTypeError, "normal must be of type Geom::Vector3d and must be valid." unless normal.instance_of?(Geom::Vector3d) || !normal.valid?

      @vertex = vertex
      @normal = normal.normalize
    end

    # @param pt [Geom::Point3d]
    # @return [Integer]
    def classify_point(pt)
      if pt.on_plane?([vertex.position, normal])
        Sketchup::Face::PointOnFace | Sketchup::Face::PointInside
      else
        Sketchup::Face::PointNotOnPlane
      end
    end

    # @return [Array<Geom::Point3d, Geom::Vector3d>]
    def plane
      [@vertex.position, @normal.clone]
    end

    # @return [Array<Geom::Point3d>]
    def vertices
      plane = self.plane
      verts = [@vertex]
      @vertex.edges.each do |edge|
        other_vert = edge.other_vertex(@vertex)
        next unless other_vert.position.on_plane?(plane)

        verts << other_vert
      end

      verts
    end

    # Filler method to avoid errors
    # @return [Nil]
    def material
      nil
    end

    # Filler method to avoid errors
    # @return [Nil]
    def back_material
      nil
    end

    def inspect
      self.to_s
    end

  end

end