
module DM::Bevel

  class BevelerDataManager


    # @return [Hash {Sketchup::Face => FaceLoops}]
    attr_reader :face_to_face_loops
    # @return [Array<FaceCreateData>]
    attr_reader :face_create_datas
    # @return [Array<EdgeCreateData>]
    attr_reader :force_smoothness_edge_create_datas

    # @param faces [Sketchup::Face]
    def initialize(faces)
      face_to_face_loops = {}
      faces.each do |face|
        raise UnexpectedTypeError, "Not a Sketchup::Face" unless face.instance_of?(Sketchup::Face)
        raise UnexpectedStateError, "Duplicate Faces found" unless face_to_face_loops[face].nil?

        face_to_face_loops[face] = FaceLoops.new(face)
      end
      @face_to_face_loops = face_to_face_loops

      @face_create_datas = []
      @force_smoothness_edge_create_datas = []
    end

    # @param face_create_datas [FaceCreateData, Array<FaceCreateData>]
    def add_face_create_datas(face_create_datas)
      face_create_datas = [face_create_datas] unless face_create_datas.instance_of?(Array)

      face_create_datas.each do |face_create_data|
        raise UnexpectedTypeError, "Must be of type FaceCreateData" unless face_create_data.instance_of?(FaceCreateData)
        @face_create_datas << face_create_data
      end
    end

    # @param edge_create_datas [EdgeCreateData, Array<EdgeCreateData>]
    def add_force_smoothness_edge_create(edge_create_datas)
      edge_create_datas = [edge_create_datas] unless edge_create_datas.instance_of?(Array)

      edge_create_datas.each do |edge_create_data|
        raise UnexpectedTypeError, "Must be of type EdgeCreateData" unless edge_create_data.instance_of?(EdgeCreateData)

        @force_smoothness_edge_create_datas << edge_create_data
      end
    end

  end

end