require 'set'

module DM::Bevel

  ON_MAC = Sketchup.platform == :platform_osx
  PATH = File.dirname(__FILE__)
  MAIN_DICTIONARY = 'Bevel'

  SOURCE_FILES = %w(
    singleton
    window
    bevel_exceptions
    cache_manager
    geometry_mixin
    bevel_mixin
    debugger_mixin
    debugger_mixin_bevel
    tool/bevelable_edge_state
    tool/draw_mixin
    tool/core_tool
    tool/base_tool
    tool/bevel_previewer
    tool/edge_extruder
    calculation/fake_face
    calculation/quad_face
    calculation/edge_vertex
    calculation/face_corner/face_corner
    calculation/face_corner/coplanar_face_corner
    calculation/face_corner/fake_face_corner
    calculation/face_corner/parallel_face_corner
    calculation/face_corner/quad_face_corner
    calculation/bevel_curve
    calculation/transition_segment
    calculation/corner
    calculation/corner_patch
    calculation/bevel_edge_boundary
    calculation/collision_report
    bevel_params
    bevel_data
    bevel_limiter
    geometry/loop_vertex
    geometry/face_loops
    geometry/beveler_data
    geometry/beveler_entities
    geometry/beveler_generator
    geometry/beveler_data_manager
    proxymesh
    beveler
    bevel_proxy_settings
    bevel_non_destructive
    bevel_tool
    bevel_nd_offset_tool
    bevel_settings_dialog
    coplanar_collinear_cleaner
    soften_edges
    debug_bevel_tool
    main
    observers
  )

  # Reload is used for development only so we know we have .rb files
  #  (.rbe files cannot be reloaded with the load command)
  def self.reload
    original_verbose = $VERBOSE
    $VERBOSE = nil
    (SOURCE_FILES - ['main']).each { |source| load(File.join(PATH, source + ".rb")) }
    SKETCHUP_CONSOLE.clear
    puts "Bevel Reloaded"
  ensure
    $VERBOSE = original_verbose
  end

  def self.load_source_files
    SOURCE_FILES.each { |source| Sketchup::require(File.join(PATH, source))}
  end

  self.load_source_files


  # Code is used to prevent precision issues when comparing double values
  FLOAT_PRECISION = 5 # used to round double values to prevent precision issues

  #freeze # Freeze Modules prior to public release!!!!

end


