module DM::Bevel

  # @param [Sketchup::Edge] edge
  def self.is_bevel_perimeter?(edge)
    edge.get_attribute(MAIN_DICTIONARY, "perimeter_edge", false)
  end

  # @param [Sketchup::Entities] entities
  def soften_edges(entities = Sketchup.active_model.active_entities)
    edges = entities.grep(Sketchup::Edge)
    edges.each do |edge|
      next if QuadFace.is_quadface_diagonal?(edge) # ignore quad diagonals
      faces = edge.faces
      next if faces.length != 2 # ignore edges without exactly 2 faces

      angle = faces[0].normal.angle_between(faces[1].normal) # radians
      if angle > @bevel_params.soften_angle.degrees
        # edge should be hard (soft = false)
        edge.soft = false
        edge.smooth = PLUGIN.is_bevel_perimeter?(edge) ? @bevel_params.smooth_perimeter? : false
      else
        edge.soft = true
        edge.smooth = PLUGIN.is_bevel_perimeter?(edge) ? @bevel_params.smooth_perimeter? : true
      end
    end
  end

  # @param entities [Array<Sketchup::Entity>]
  def self.soften_edges(entities)
    if entities == []
      entities = Sketchup.active_model.active_entities
      no_selection = true
    end
    model = Sketchup.active_model
    edges = []
    entities.each do |ent|
      if ent.instance_of?(Sketchup::Edge)
        edges << ent
      elsif ent.instance_of?(Sketchup::Face)
        edges.concat(ent.edges)
      end
    end
    edges.uniq!
    softened = 0
    begin
      raise EmptySelectionError, LH["No edges found. Selection must contain edges."] if edges.empty?

      smooth_perimeter = Sketchup.read_default(MAIN_DICTIONARY, BevelSettingsDialog::ID_SMOOTH_PERIMETER, BevelParams::Defaults::SMOOTH_PERIMETER)
      soften_angle = Sketchup.read_default(MAIN_DICTIONARY, BevelSettingsDialog::ID_SOFTEN_ANGLE, BevelParams::Defaults::SOFTEN_ANGLE)
      smooth_normals = Sketchup.read_default(MAIN_DICTIONARY, BevelSettingsDialog::ID_SMOOTH_NORMALS, BevelParams::Defaults::SMOOTH_NORMALS)

      model.start_operation(LH["Soften Edges"], true)

      edges.each do |edge|
        next if QuadFace.is_quadface_diagonal?(edge) # ignore quad diagonals
        faces = edge.faces
        next if faces.length != 2 # ignore edges without exactly 2 faces

        angle = faces[0].normal.angle_between(faces[1].normal) # radians
        if angle > soften_angle.degrees
          # edge should be hard (soft = false)
          edge.soft = false
          edge.smooth = false
        else
          edge.soft = true
          softened += 1
          perimeter_edge = PLUGIN.is_bevel_perimeter?(edge)
          edge.smooth = perimeter_edge ? smooth_perimeter : smooth_normals
        end
      end

      model.commit_operation
      self.notify_soften_complete(edges, softened, no_selection)
    rescue EmptySelectionError => err
      UI.messagebox(err.message)
      model.abort_operation
    end
  end

  def self.notify_soften_complete(edges, softened, no_selection = false)
    # Prior to 2019 if button is enabled 2 buttons will always be displayed
    message = LH["Auto-Soften Edges"] + "\r\n\r\n"
    message += no_selection ? LH["Processed All Edges in Context: "] : LH["Edges Selected: "]
    message += "#{edges.length}\r\n"
    message += LH["Edges Softened: "]
    message += "#{softened}"
    PLUGIN.notification(message, "bevel-soft_icon.png")
  end

end