
module DM::Bevel

  class BevelPreviewer
    include DrawMixin

    DRAWING_COLOR = Sketchup::Color.new('orange')

    def initialize
      # @type [BevelData]
      @bevel_data = nil
    end

    # @param bevel_data [BevelData]
    def data=(bevel_data)
      raise UnexpectedTypeError, "Must be of type BevelData" unless bevel_data.instance_of?(BevelData)
      # @type [BevelData]
      @bevel_data = bevel_data
    end

    # @param view [Sketchup::View]
    # @param low_detail [Boolean]
    def draw(view, low_detail = false)
      # @type [BevelData]
      bevel_data = @bevel_data
      return if bevel_data.none_calculated?

      view.drawing_color = DRAWING_COLOR
      all_lines = []

      if bevel_data.edge_offset_too_small?
        # If it's too small, just draw the primary edges.
        bevel_data.primary_edges.each do |edge|
          all_lines << [edge.start.position, edge.end.position]
        end
      else
        # To avoid drawing the same line (but in the opposite direction) track it with a hash.
        _drawn_data = {}
        bevel_data.each_transition.each do |transition_segment|
          transition_segment.face_to_opposite_start_end_pts.values.each do |pt, opposite_pt|
            next unless _drawn_data[[pt, opposite_pt]].nil?

            all_lines << [pt, opposite_pt]

            _drawn_data[[pt, opposite_pt]] = true
            _drawn_data[[opposite_pt, pt]] = true
          end

          next if low_detail

          transition_segment.adjacent_transition_segments.each do |adjacent_transition_segment|
            next unless _drawn_data[adjacent_transition_segment].nil?
            next unless adjacent_transition_segment.three_point_segment?

            all_lines << adjacent_transition_segment.curve.points
            _drawn_data[adjacent_transition_segment] = true
          end

          all_lines << transition_segment.curve.points
          _drawn_data[transition_segment] = true
        end

      end
      draw_2d_lines(view, all_lines,size: 2, stipple: "" )
    end

  end
end
