module DM::Bevel

  module DrawMixin

    # @param view [Sketchup::View]
    # @param line [Array<Geom::Point3d>]
    # @param data [Hash]
    # @option :color [String]
    # @option :stipple [String]
    # @option :size [Integer]
    def draw_2d_line(view, line, **data)
      view.drawing_color = data[:color] unless data[:color].nil?
      view.line_stipple = data[:stipple].to_s
      view.line_width = data[:size].to_i
      line = line.map {|pt| view.screen_coords(pt)}
      view.draw2d(GL_LINE_STRIP, line)
      view.line_stipple = '' # reset it
    end


    # @param view [Sketchup::View]
    # @param lines [Array<Array<Geom::Point3d>>]
    # @param data [Hash]
    # @option :color [String]
    # @option :stipple [String]
    # @option :size [Integer]
    def draw_2d_lines(view, lines, **data)
      view.drawing_color = data[:color] unless data[:color].nil?
      view.line_stipple = data[:stipple].to_s
      view.line_width = data[:size].to_i

      all_lines = []
      lines.each do |line_pts|
        (line_pts.length - 1).times do |pt_i|
          all_lines << view.screen_coords(line_pts[pt_i])
          all_lines << view.screen_coords(line_pts[pt_i + 1])
        end
      end
      view.draw2d(GL_LINES, all_lines)
      view.line_stipple = '' # reset it
    end

  end

end